@@ -1,5 +1,5 @@
-use crate::generator::{utils::*, LuaGenerator};
-use crate::nodes;
+use crate::generator::{utils, LuaGenerator};
+use crate::nodes::{self, Identifier};
 
 /// This implementation of [LuaGenerator](trait.LuaGenerator.html) attempts to produce Lua code as
 /// small as possible. It is not meant to be read by humans.
@@ -108,12 +108,12 @@ impl DenseLuaGenerator {
 
     #[inline]
     fn needs_space(&self, next_character: char) -> bool {
-        is_relevant_for_spacing(&next_character)
+        utils::is_relevant_for_spacing(&next_character)
             && self
                 .output
                 .chars()
                 .last()
-                .filter(is_relevant_for_spacing)
+                .filter(utils::is_relevant_for_spacing)
                 .is_some()
     }
 
@@ -160,11 +160,11 @@ impl DenseLuaGenerator {
             .unwrap_or("")
     }
 
-    fn write_function_parameters(&mut self, parameters: &[String], is_variadic: bool) {
+    fn write_function_parameters(&mut self, parameters: &[Identifier], is_variadic: bool) {
         let last_index = parameters.len().saturating_sub(1);
 
         parameters.iter().enumerate().for_each(|(index, variable)| {
-            self.push_str(variable);
+            self.push_str(variable.get_name());
 
             if index != last_index {
                 self.push_char(',');
@@ -178,15 +178,6 @@ impl DenseLuaGenerator {
             self.push_str("...");
         };
     }
-
-    fn write_variable(&mut self, variable: &nodes::Variable) {
-        use nodes::Variable::*;
-        match variable {
-            Identifier(identifier) => self.push_str(identifier),
-            Field(field) => self.write_field(field),
-            Index(index) => self.write_index(index),
-        }
-    }
 }
 
 impl Default for DenseLuaGenerator {
@@ -209,7 +200,9 @@ impl LuaGenerator for DenseLuaGenerator {
             self.write_statement(statement);
 
             if let Some(next_statement) = statements.peek() {
-                if starts_with_parenthese(next_statement) && ends_with_prefix(statement) {
+                if utils::starts_with_parenthese(next_statement)
+                    && utils::ends_with_prefix(statement)
+                {
                     self.push_char(';');
                 }
             }
@@ -261,7 +254,7 @@ impl LuaGenerator for DenseLuaGenerator {
             .iter()
             .enumerate()
             .for_each(|(index, identifier)| {
-                self.push_str(identifier);
+                self.push_str(identifier.get_name());
 
                 if index != last_identifier_index {
                     self.push_char(',');
@@ -314,15 +307,15 @@ impl LuaGenerator for DenseLuaGenerator {
         self.push_str("function");
         let name = function.get_name();
 
-        self.push_str(name.get_name());
+        self.push_str(name.get_name().get_name());
         name.get_field_names().iter().for_each(|field| {
             self.push_char('.');
-            self.push_str(field);
+            self.push_str(field.get_name());
         });
 
         if let Some(method) = name.get_method() {
             self.push_char(':');
-            self.push_str(method);
+            self.push_str(method.get_name());
         }
 
         self.push_char('(');
@@ -341,14 +334,14 @@ impl LuaGenerator for DenseLuaGenerator {
         use nodes::LastStatement::*;
 
         match statement {
-            Break => self.push_str("break"),
-            Continue => self.push_str("continue"),
+            Break(_) => self.push_str("break"),
+            Continue(_) => self.push_str("continue"),
             Return(expressions) => {
                 self.push_str("return");
                 let last_index = expressions.len().saturating_sub(1);
 
                 expressions
-                    .iter()
+                    .iter_expressions()
                     .enumerate()
                     .for_each(|(index, expression)| {
                         self.write_expression(expression);
@@ -368,7 +361,7 @@ impl LuaGenerator for DenseLuaGenerator {
         let last_variable_index = variables.len().saturating_sub(1);
 
         variables.iter().enumerate().for_each(|(index, variable)| {
-            self.push_str(variable);
+            self.push_str(variable.get_name());
 
             if index != last_variable_index {
                 self.push_char(',');
@@ -420,7 +413,7 @@ impl LuaGenerator for DenseLuaGenerator {
     fn write_numeric_for(&mut self, numeric_for: &nodes::NumericForStatement) {
         self.push_str("for");
 
-        self.push_str(numeric_for.get_identifier());
+        self.push_str(numeric_for.get_identifier().get_name());
         self.push_char('=');
         self.write_expression(numeric_for.get_start());
         self.push_char(',');
@@ -475,24 +468,20 @@ impl LuaGenerator for DenseLuaGenerator {
         match expression {
             Binary(binary) => self.write_binary_expression(binary),
             Call(call) => self.write_function_call(call),
-            False => self.push_str("false"),
+            False(_) => self.push_str("false"),
             Field(field) => self.write_field(field),
             Function(function) => self.write_function(function),
-            Identifier(identifier) => self.push_str(identifier),
+            Identifier(identifier) => self.write_identifier(identifier),
             Index(index) => self.write_index(index),
-            Nil => self.push_str("nil"),
+            Nil(_) => self.push_str("nil"),
             Number(number) => self.write_number(number),
-            Parenthese(expression) => {
-                self.push_char('(');
-                self.write_expression(expression);
-                self.push_char(')');
-            }
+            Parenthese(parenthese) => self.write_parenthese(parenthese),
             String(string) => self.write_string(string),
             Table(table) => self.write_table(table),
-            True => self.push_str("true"),
+            True(_) => self.push_str("true"),
             Unary(unary) => self.write_unary_expression(unary),
-            VariableArguments => {
-                self.push_str_and_break_if("...", break_variable_arguments);
+            VariableArguments(_) => {
+                self.push_str_and_break_if("...", utils::break_variable_arguments);
             }
         }
     }
@@ -513,7 +502,7 @@ impl LuaGenerator for DenseLuaGenerator {
         }
 
         match operator {
-            BinaryOperator::Concat => self.push_str_and_break_if("..", break_concat),
+            BinaryOperator::Concat => self.push_str_and_break_if("..", utils::break_concat),
             _ => self.push_str(operator.to_str()),
         }
 
@@ -531,7 +520,7 @@ impl LuaGenerator for DenseLuaGenerator {
 
         match unary.operator() {
             Length => self.push_char('#'),
-            Minus => self.push_str_and_break_if("-", break_minus),
+            Minus => self.push_str_and_break_if("-", utils::break_minus),
             Not => self.push_str("not"),
         }
 
@@ -568,7 +557,7 @@ impl LuaGenerator for DenseLuaGenerator {
 
         if let Some(method) = &call.get_method() {
             self.push_char(':');
-            self.push_str(method);
+            self.push_str(method.get_name());
         }
 
         self.write_arguments(call.get_arguments());
@@ -578,7 +567,7 @@ impl LuaGenerator for DenseLuaGenerator {
         self.write_prefix(field.get_prefix());
 
         self.push_char('.');
-        self.push_str(field.get_field());
+        self.push_str(field.get_field().get_name());
     }
 
     fn write_index(&mut self, index: &nodes::IndexExpression) {
@@ -589,22 +578,6 @@ impl LuaGenerator for DenseLuaGenerator {
         self.push_char(']');
     }
 
-    fn write_prefix(&mut self, prefix: &nodes::Prefix) {
-        use nodes::Prefix::*;
-
-        match prefix {
-            Call(call) => self.write_function_call(call),
-            Field(field) => self.write_field(field),
-            Identifier(identifier) => self.push_str(identifier),
-            Index(index) => self.write_index(index),
-            Parenthese(expression) => {
-                self.push_char('(');
-                self.write_expression(expression);
-                self.push_char(')');
-            }
-        }
-    }
-
     fn write_table(&mut self, table: &nodes::TableExpression) {
         self.push_char('{');
 
@@ -623,22 +596,20 @@ impl LuaGenerator for DenseLuaGenerator {
     }
 
     fn write_table_entry(&mut self, entry: &nodes::TableEntry) {
-        use nodes::TableEntry::*;
-
         match entry {
-            Field(identifier, value) => {
-                self.push_str(identifier);
+            nodes::TableEntry::Field(entry) => {
+                self.push_str(entry.get_field().get_name());
                 self.push_char('=');
-                self.write_expression(value);
+                self.write_expression(entry.get_value());
             }
-            Index(key, value) => {
+            nodes::TableEntry::Index(entry) => {
                 self.push_char('[');
-                self.write_expression(key);
+                self.write_expression(entry.get_key());
                 self.push_char(']');
                 self.push_char('=');
-                self.write_expression(value);
+                self.write_expression(entry.get_value());
             }
-            Value(expression) => self.write_expression(expression),
+            nodes::TableEntry::Value(expression) => self.write_expression(expression),
         }
     }
 
@@ -708,72 +679,40 @@ impl LuaGenerator for DenseLuaGenerator {
         }
     }
 
-    fn write_arguments(&mut self, arguments: &nodes::Arguments) {
-        use nodes::Arguments::*;
-        match arguments {
-            String(string) => self.write_string(string),
-            Table(table) => self.write_table(table),
-            Tuple(expressions) => {
-                self.merge_char('(');
+    fn write_tuple_arguments(&mut self, arguments: &nodes::TupleArguments) {
+        self.merge_char('(');
 
-                let last_index = expressions.len().saturating_sub(1);
-                expressions
-                    .iter()
-                    .enumerate()
-                    .for_each(|(index, expression)| {
-                        self.write_expression(expression);
+        let last_index = arguments.len().saturating_sub(1);
+        arguments
+            .iter_values()
+            .enumerate()
+            .for_each(|(index, expression)| {
+                self.write_expression(expression);
 
-                        if index != last_index {
-                            self.push_char(',');
-                        }
-                    });
+                if index != last_index {
+                    self.push_char(',');
+                }
+            });
 
-                self.push_char(')');
-            }
-        }
+        self.push_char(')');
     }
 
     fn write_string(&mut self, string: &nodes::StringExpression) {
-        let value = string.get_value();
-        if string.is_multiline() {
-            let mut i = 0;
-            let mut equals = "=".repeat(i);
-
-            loop {
-                if !value.contains(&format!("]{}]", equals)) {
-                    break;
-                } else {
-                    i += 1;
-                    equals = "=".repeat(i);
-                };
-            }
-
-            self.push_str_and_break_if(
-                &format!("[{}[{}]{}]", equals, value, equals),
-                break_long_string,
-            );
+        let result = utils::write_string(string);
+        if result.starts_with('[') {
+            self.push_str_and_break_if(&result, utils::break_long_string);
         } else {
-            let string = if string.has_single_quote() {
-                if string.has_double_quote() {
-                    let mut total_escaped = 0;
-                    let mut escaped_string = value.to_owned();
+            self.push_str(&result);
+        }
+    }
 
-                    let mut chars = value.char_indices();
+    fn write_identifier(&mut self, identifier: &nodes::Identifier) {
+        self.push_str(identifier.get_name());
+    }
 
-                    while let Some(unescaped_index) = find_not_escaped_from('\'', &mut chars) {
-                        escaped_string.insert(unescaped_index + total_escaped, '\\');
-                        total_escaped += 1;
-                    }
-
-                    format!("'{}'", escaped_string)
-                } else {
-                    format!("\"{}\"", value)
-                }
-            } else {
-                format!("'{}'", value)
-            };
-
-            self.push_str(&string);
-        };
+    fn write_parenthese(&mut self, parenthese: &nodes::ParentheseExpression) {
+        self.push_char('(');
+        self.write_expression(parenthese.inner_expression());
+        self.push_char(')');
     }
 }
