@@ -1,5 +1,5 @@
-use crate::generator::{utils::*, LuaGenerator};
-use crate::nodes;
+use crate::generator::{utils, LuaGenerator};
+use crate::nodes::{self, Identifier};
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
 enum StatementType {
@@ -44,8 +44,8 @@ impl From<&nodes::LastStatement> for StatementType {
     fn from(statement: &nodes::LastStatement) -> Self {
         use nodes::LastStatement::*;
         match statement {
-            Break => Self::Break,
-            Continue => Self::Continue,
+            Break(_) => Self::Break,
+            Continue(_) => Self::Continue,
             Return(_) => Self::Return,
         }
     }
@@ -159,12 +159,12 @@ impl ReadableLuaGenerator {
 
     #[inline]
     fn needs_space(&self, next_character: char) -> bool {
-        is_relevant_for_spacing(&next_character)
+        utils::is_relevant_for_spacing(&next_character)
             && self
                 .output
                 .chars()
                 .last()
-                .filter(is_relevant_for_spacing)
+                .filter(utils::is_relevant_for_spacing)
                 .is_some()
     }
 
@@ -236,9 +236,10 @@ impl ReadableLuaGenerator {
             })
             || entries.len() == 1
                 && entries.iter().all(|entry| match entry {
-                    TableEntry::Field(_identifier, value) => self.is_small_expression(value),
-                    TableEntry::Index(key, value) => {
-                        self.is_small_expression(key) && self.is_small_expression(value)
+                    TableEntry::Field(entry) => self.is_small_expression(entry.get_value()),
+                    TableEntry::Index(entry) => {
+                        self.is_small_expression(entry.get_key())
+                            && self.is_small_expression(entry.get_value())
                     }
                     _ => false,
                 })
@@ -247,16 +248,16 @@ impl ReadableLuaGenerator {
     fn is_small_expression(&self, expression: &nodes::Expression) -> bool {
         use nodes::Expression::*;
         match expression {
-            True | False | Nil | Identifier(_) | VariableArguments | Number(_) => true,
+            True(_) | False(_) | Nil(_) | Identifier(_) | VariableArguments(_) | Number(_) => true,
             Table(table) => table.is_empty(),
             _ => false,
         }
     }
 
-    fn write_function_parameters(&mut self, parameters: &[String], is_variadic: bool) {
+    fn write_function_parameters(&mut self, parameters: &[Identifier], is_variadic: bool) {
         let mut parameters_length = parameters
             .iter()
-            .fold(0, |acc, parameter| acc + parameter.len());
+            .fold(0, |acc, parameter| acc + parameter.get_name().len());
         // add a comma and a space between each parameter
         parameters_length += parameters.len() * 2;
 
@@ -273,7 +274,7 @@ impl ReadableLuaGenerator {
 
         if self.fits_on_current_line(parameters_length) {
             parameters.iter().enumerate().for_each(|(index, variable)| {
-                self.raw_push_str(variable);
+                self.raw_push_str(variable.get_name());
 
                 if index != last_index {
                     self.raw_push_char(',');
@@ -294,7 +295,7 @@ impl ReadableLuaGenerator {
             parameters.iter().enumerate().for_each(|(index, variable)| {
                 self.push_new_line();
                 self.write_indentation();
-                self.raw_push_str(variable);
+                self.raw_push_str(variable.get_name());
 
                 if index != last_index {
                     self.raw_push_char(',');
@@ -319,7 +320,7 @@ impl ReadableLuaGenerator {
     fn write_variable(&mut self, variable: &nodes::Variable) {
         use nodes::Variable::*;
         match variable {
-            Identifier(identifier) => self.push_str(identifier),
+            Identifier(identifier) => self.push_str(identifier.get_name()),
             Field(field) => self.write_field(field),
             Index(index) => self.write_index(index),
         }
@@ -348,7 +349,9 @@ impl LuaGenerator for ReadableLuaGenerator {
             self.write_statement(statement);
 
             if let Some(next_statement) = statements.peek() {
-                if starts_with_parenthese(next_statement) && ends_with_prefix(statement) {
+                if utils::starts_with_parenthese(next_statement)
+                    && utils::ends_with_prefix(statement)
+                {
                     self.push_char(';');
                 }
 
@@ -374,15 +377,15 @@ impl LuaGenerator for ReadableLuaGenerator {
         use nodes::LastStatement::*;
 
         match statement {
-            Break => self.push_str("break"),
-            Continue => self.push_str("continue"),
+            Break(_) => self.push_str("break"),
+            Continue(_) => self.push_str("continue"),
             Return(expressions) => {
                 self.push_str("return");
                 self.push_can_add_new_line(false);
                 let last_index = expressions.len().saturating_sub(1);
 
                 expressions
-                    .iter()
+                    .iter_expressions()
                     .enumerate()
                     .for_each(|(index, expression)| {
                         self.write_expression(expression);
@@ -437,7 +440,7 @@ impl LuaGenerator for ReadableLuaGenerator {
         let last_variable_index = variables.len().saturating_sub(1);
 
         variables.iter().enumerate().for_each(|(index, variable)| {
-            self.raw_push_str(variable);
+            self.raw_push_str(variable.get_name());
 
             if index != last_variable_index {
                 self.raw_push_char(',');
@@ -508,7 +511,7 @@ impl LuaGenerator for ReadableLuaGenerator {
             .iter()
             .enumerate()
             .for_each(|(index, identifier)| {
-                self.raw_push_str(identifier);
+                self.raw_push_str(identifier.get_name());
 
                 if index != last_identifier_index {
                     self.raw_push_char(',');
@@ -547,7 +550,7 @@ impl LuaGenerator for ReadableLuaGenerator {
     fn write_numeric_for(&mut self, numeric_for: &nodes::NumericForStatement) {
         self.push_str("for ");
 
-        self.raw_push_str(numeric_for.get_identifier());
+        self.raw_push_str(numeric_for.get_identifier().get_name());
         self.raw_push_char('=');
         self.write_expression(numeric_for.get_start());
         self.raw_push_char(',');
@@ -601,15 +604,15 @@ impl LuaGenerator for ReadableLuaGenerator {
         self.push_str("function ");
         let name = function.get_name();
 
-        self.raw_push_str(name.get_name());
+        self.raw_push_str(name.get_name().get_name());
         name.get_field_names().iter().for_each(|field| {
             self.raw_push_char('.');
-            self.raw_push_str(field);
+            self.raw_push_str(field.get_name());
         });
 
         if let Some(method) = name.get_method() {
             self.raw_push_char(':');
-            self.raw_push_str(method);
+            self.raw_push_str(method.get_name());
         }
 
         self.raw_push_char('(');
@@ -679,26 +682,20 @@ impl LuaGenerator for ReadableLuaGenerator {
         match expression {
             Binary(binary) => self.write_binary_expression(binary),
             Call(call) => self.write_function_call(call),
-            False => self.push_str("false"),
+            False(_) => self.push_str("false"),
             Field(field) => self.write_field(field),
             Function(function) => self.write_function(function),
-            Identifier(identifier) => self.push_str(identifier),
+            Identifier(identifier) => self.write_identifier(identifier),
             Index(index) => self.write_index(index),
-            Nil => self.push_str("nil"),
+            Nil(_) => self.push_str("nil"),
             Number(number) => self.write_number(number),
-            Parenthese(expression) => {
-                self.push_char('(');
-                self.push_can_add_new_line(false);
-                self.write_expression(expression);
-                self.pop_can_add_new_line();
-                self.push_char(')');
-            }
+            Parenthese(parenthese) => self.write_parenthese(parenthese),
             String(string) => self.write_string(string),
             Table(table) => self.write_table(table),
-            True => self.push_str("true"),
+            True(_) => self.push_str("true"),
             Unary(unary) => self.write_unary_expression(unary),
-            VariableArguments => {
-                self.push_str_and_break_if("...", break_variable_arguments);
+            VariableArguments(_) => {
+                self.push_str_and_break_if("...", utils::break_variable_arguments);
             }
         }
     }
@@ -734,7 +731,7 @@ impl LuaGenerator for ReadableLuaGenerator {
 
         match unary.operator() {
             Length => self.push_char('#'),
-            Minus => self.push_str_and_break_if("-", break_minus),
+            Minus => self.push_str_and_break_if("-", utils::break_minus),
             Not => self.push_str("not "),
         }
 
@@ -774,7 +771,7 @@ impl LuaGenerator for ReadableLuaGenerator {
 
         if let Some(method) = &call.get_method() {
             self.push_char(':');
-            self.push_str(method);
+            self.push_str(method.get_name());
         }
 
         self.write_arguments(call.get_arguments());
@@ -782,30 +779,23 @@ impl LuaGenerator for ReadableLuaGenerator {
         self.pop_can_add_new_line();
     }
 
-    fn write_arguments(&mut self, arguments: &nodes::Arguments) {
-        use nodes::Arguments::*;
-        match arguments {
-            String(string) => self.write_string(string),
-            Table(table) => self.write_table(table),
-            Tuple(expressions) => {
-                self.raw_push_char('(');
+    fn write_tuple_arguments(&mut self, arguments: &nodes::TupleArguments) {
+        self.raw_push_char('(');
 
-                let last_index = expressions.len().saturating_sub(1);
-                expressions
-                    .iter()
-                    .enumerate()
-                    .for_each(|(index, expression)| {
-                        self.write_expression(expression);
+        let last_index = arguments.len().saturating_sub(1);
+        arguments
+            .iter_values()
+            .enumerate()
+            .for_each(|(index, expression)| {
+                self.write_expression(expression);
 
-                        if index != last_index {
-                            self.raw_push_char(',');
-                            self.raw_push_char(' ');
-                        }
-                    });
+                if index != last_index {
+                    self.raw_push_char(',');
+                    self.raw_push_char(' ');
+                }
+            });
 
-                self.push_char(')');
-            }
-        }
+        self.push_char(')');
     }
 
     fn write_field(&mut self, field: &nodes::FieldExpression) {
@@ -814,7 +804,7 @@ impl LuaGenerator for ReadableLuaGenerator {
         self.pop_can_add_new_line();
 
         self.push_char('.');
-        self.raw_push_str(field.get_field());
+        self.raw_push_str(field.get_field().get_name());
     }
 
     fn write_index(&mut self, index: &nodes::IndexExpression) {
@@ -829,26 +819,6 @@ impl LuaGenerator for ReadableLuaGenerator {
         self.pop_can_add_new_line();
     }
 
-    fn write_prefix(&mut self, prefix: &nodes::Prefix) {
-        use nodes::Prefix::*;
-
-        match prefix {
-            Call(call) => self.write_function_call(call),
-            Field(field) => self.write_field(field),
-            Identifier(identifier) => self.push_str(identifier),
-            Index(index) => self.write_index(index),
-            Parenthese(expression) => {
-                self.push_char('(');
-                self.push_can_add_new_line(false);
-
-                self.write_expression(expression);
-
-                self.pop_can_add_new_line();
-                self.push_char(')');
-            }
-        }
-    }
-
     fn write_table(&mut self, table: &nodes::TableExpression) {
         self.push_char('{');
 
@@ -890,120 +860,48 @@ impl LuaGenerator for ReadableLuaGenerator {
     }
 
     fn write_table_entry(&mut self, entry: &nodes::TableEntry) {
-        use nodes::TableEntry::*;
-
         match entry {
-            Field(identifier, value) => {
-                self.raw_push_str(identifier);
+            nodes::TableEntry::Field(entry) => {
+                self.raw_push_str(entry.get_field().get_name());
                 self.raw_push_str(" = ");
-                self.write_expression(value);
+                self.write_expression(entry.get_value());
             }
-            Index(key, value) => {
+            nodes::TableEntry::Index(entry) => {
                 self.raw_push_char('[');
                 self.push_can_add_new_line(false);
-                self.write_expression(key);
+                self.write_expression(entry.get_key());
                 self.pop_can_add_new_line();
                 self.raw_push_str("] = ");
-                self.write_expression(value);
+                self.write_expression(entry.get_value());
             }
-            Value(expression) => self.write_expression(expression),
+            nodes::TableEntry::Value(expression) => self.write_expression(expression),
         }
     }
 
     fn write_number(&mut self, number: &nodes::NumberExpression) {
-        use nodes::NumberExpression::*;
-
-        match number {
-            Decimal(number) => {
-                let float = number.get_raw_float();
-                if float.is_nan() {
-                    self.push_str("(0/0)");
-                } else if float.is_infinite() {
-                    self.push_char('(');
-                    if float.is_sign_negative() {
-                        self.push_char('-');
-                    }
-                    self.push_str("1/0)")
-                } else {
-                    self.push_str(&format!("{:.}", float));
-
-                    if let Some(exponent) = number.get_exponent() {
-                        let exponent_char = number
-                            .is_uppercase()
-                            .map(|is_uppercase| if is_uppercase { 'E' } else { 'e' })
-                            .unwrap_or('e');
-                        self.raw_push_char(exponent_char);
-                        self.raw_push_str(&format!("{}", exponent));
-                    };
-                }
-            }
-            Hex(number) => {
-                self.push_str(&format!(
-                    "0{}{:x}",
-                    if number.is_x_uppercase() { 'X' } else { 'x' },
-                    number.get_raw_integer()
-                ));
-
-                if let Some(exponent) = number.get_exponent() {
-                    let exponent_char = number
-                        .is_exponent_uppercase()
-                        .map(|is_uppercase| if is_uppercase { 'P' } else { 'p' })
-                        .unwrap_or('p');
-                    self.raw_push_char(exponent_char);
-                    self.raw_push_str(&format!("{}", exponent));
-                };
-            }
-            Binary(number) => {
-                self.push_str(&format!(
-                    "0{}{:b}",
-                    if number.is_b_uppercase() { 'B' } else { 'b' },
-                    number.get_raw_value()
-                ));
-            }
-        }
+        self.push_str(&utils::write_number(number));
     }
 
     fn write_string(&mut self, string: &nodes::StringExpression) {
-        let value = string.get_value();
-        if string.is_multiline() {
-            let mut i = 0;
-            let mut equals = "=".repeat(i);
-
-            loop {
-                if !value.contains(&format!("]{}]", equals)) {
-                    break;
-                } else {
-                    i += 1;
-                    equals = "=".repeat(i);
-                };
-            }
-
-            self.push_str_and_break_if(
-                &format!("[{}[{}]{}]", equals, value, equals),
-                break_long_string,
-            );
+        let result = utils::write_string(string);
+        if result.starts_with('[') {
+            self.push_str_and_break_if(&result, utils::break_long_string);
         } else {
-            let string = if string.has_single_quote() {
-                if string.has_double_quote() {
-                    let mut total_escaped = 0;
-                    let mut escaped_string = value.to_owned();
+            self.push_str(&result);
+        }
+    }
 
-                    let mut chars = value.char_indices();
+    fn write_identifier(&mut self, identifier: &nodes::Identifier) {
+        self.push_str(identifier.get_name());
+    }
 
-                    while let Some(unescaped_index) = find_not_escaped_from('\'', &mut chars) {
-                        escaped_string.insert(unescaped_index + total_escaped, '\\');
-                        total_escaped += 1;
-                    }
+    fn write_parenthese(&mut self, parenthese: &nodes::ParentheseExpression) {
+        self.push_char('(');
+        self.push_can_add_new_line(false);
 
-                    format!("'{}'", escaped_string)
-                } else {
-                    format!("\"{}\"", value)
-                }
-            } else {
-                format!("'{}'", value)
-            };
+        self.write_expression(parenthese.inner_expression());
 
-            self.push_str(&string);
-        };
+        self.pop_can_add_new_line();
+        self.push_char(')');
     }
 }
