@@ -1,9 +1,16 @@
-use crate::nodes::{LastStatement, Statement};
+use crate::nodes::{LastStatement, ReturnStatement, Statement, Token};
+
+#[derive(Clone, Debug, PartialEq, Eq)]
+pub struct BlockTokens {
+    pub semicolons: Vec<Option<Token>>,
+    pub last_semicolon: Option<Token>,
+}
 
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub struct Block {
     statements: Vec<Statement>,
     last_statement: Option<LastStatement>,
+    tokens: Option<Box<BlockTokens>>,
 }
 
 impl Block {
@@ -11,9 +18,25 @@ impl Block {
         Self {
             statements,
             last_statement,
+            tokens: None,
         }
     }
 
+    pub fn with_tokens(mut self, tokens: BlockTokens) -> Self {
+        self.tokens = Some(tokens.into());
+        self
+    }
+
+    #[inline]
+    pub fn set_tokens(&mut self, tokens: BlockTokens) {
+        self.tokens = Some(tokens.into());
+    }
+
+    #[inline]
+    pub fn get_tokens(&self) -> Option<&BlockTokens> {
+        self.tokens.as_ref().map(|tokens| tokens.as_ref())
+    }
+
     pub fn with_statement<T: Into<Statement>>(mut self, statement: T) -> Self {
         self.statements.push(statement.into());
         self
@@ -88,6 +111,12 @@ impl From<LastStatement> for Block {
     }
 }
 
+impl From<ReturnStatement> for Block {
+    fn from(statement: ReturnStatement) -> Block {
+        Block::new(Vec::new(), Some(statement.into()))
+    }
+}
+
 #[cfg(test)]
 mod test {
     use super::*;
@@ -109,7 +138,7 @@ mod test {
 
     #[test]
     fn is_empty_is_false_when_block_has_a_last_statement() {
-        let block = Block::default().with_last_statement(LastStatement::Break);
+        let block = Block::default().with_last_statement(LastStatement::new_break());
 
         assert!(!block.is_empty());
     }
@@ -131,7 +160,7 @@ mod test {
 
     #[test]
     fn clear_removes_last_statement() {
-        let mut block = Block::default().with_last_statement(LastStatement::Break);
+        let mut block = Block::default().with_last_statement(LastStatement::new_break());
         block.clear();
 
         assert!(block.is_empty());
