@@ -1,10 +1,13 @@
 use std::fmt::{Display, Formatter, Result as FmtResult};
 use std::str::FromStr;
 
+use crate::nodes::Token;
+
 #[derive(Clone, Debug, PartialEq)]
 pub struct DecimalNumber {
     float: f64,
     exponent: Option<(i64, bool)>,
+    token: Option<Token>,
 }
 
 impl Eq for DecimalNumber {}
@@ -14,9 +17,25 @@ impl DecimalNumber {
         Self {
             float: value,
             exponent: None,
+            token: None,
         }
     }
 
+    pub fn with_token(mut self, token: Token) -> Self {
+        self.token = Some(token);
+        self
+    }
+
+    #[inline]
+    pub fn set_token(&mut self, token: Token) {
+        self.token = Some(token);
+    }
+
+    #[inline]
+    fn get_token(&self) -> Option<&Token> {
+        self.token.as_ref()
+    }
+
     pub fn with_exponent(mut self, exponent: i64, is_uppercase: bool) -> Self {
         self.exponent.replace((exponent, is_uppercase));
         self
@@ -56,6 +75,7 @@ pub struct HexNumber {
     integer: u64,
     exponent: Option<(u32, bool)>,
     is_x_uppercase: bool,
+    token: Option<Token>,
 }
 
 impl HexNumber {
@@ -64,9 +84,25 @@ impl HexNumber {
             integer,
             exponent: None,
             is_x_uppercase,
+            token: None,
         }
     }
 
+    pub fn with_token(mut self, token: Token) -> Self {
+        self.token = Some(token);
+        self
+    }
+
+    #[inline]
+    pub fn set_token(&mut self, token: Token) {
+        self.token = Some(token);
+    }
+
+    #[inline]
+    fn get_token(&self) -> Option<&Token> {
+        self.token.as_ref()
+    }
+
     pub fn with_exponent(mut self, exponent: u32, is_uppercase: bool) -> Self {
         self.exponent.replace((exponent, is_uppercase));
         self
@@ -110,6 +146,7 @@ impl HexNumber {
 pub struct BinaryNumber {
     value: u64,
     is_b_uppercase: bool,
+    token: Option<Token>,
 }
 
 impl BinaryNumber {
@@ -117,9 +154,25 @@ impl BinaryNumber {
         Self {
             value,
             is_b_uppercase,
+            token: None,
         }
     }
 
+    pub fn with_token(mut self, token: Token) -> Self {
+        self.token = Some(token);
+        self
+    }
+
+    #[inline]
+    pub fn set_token(&mut self, token: Token) {
+        self.token = Some(token);
+    }
+
+    #[inline]
+    fn get_token(&self) -> Option<&Token> {
+        self.token.as_ref()
+    }
+
     pub fn set_uppercase(&mut self, is_uppercase: bool) {
         self.is_b_uppercase = is_uppercase;
     }
@@ -162,6 +215,33 @@ impl NumberExpression {
             Self::Binary(binary) => binary.compute_value(),
         }
     }
+
+    pub fn with_token(mut self, token: Token) -> Self {
+        match &mut self {
+            NumberExpression::Decimal(number) => number.set_token(token),
+            NumberExpression::Hex(number) => number.set_token(token),
+            NumberExpression::Binary(number) => number.set_token(token),
+        }
+        self
+    }
+
+    #[inline]
+    pub fn set_token(&mut self, token: Token) {
+        match self {
+            NumberExpression::Decimal(number) => number.set_token(token),
+            NumberExpression::Hex(number) => number.set_token(token),
+            NumberExpression::Binary(number) => number.set_token(token),
+        }
+    }
+
+    #[inline]
+    pub fn get_token(&self) -> Option<&Token> {
+        match self {
+            NumberExpression::Decimal(number) => number.get_token(),
+            NumberExpression::Hex(number) => number.get_token(),
+            NumberExpression::Binary(number) => number.get_token(),
+        }
+    }
 }
 
 impl From<DecimalNumber> for NumberExpression {
